@@ -28,6 +28,7 @@
 #include <X11/extensions/Xrandr.h>
 #include <X11/extensions/xf86vmode.h>
 #include <gconf/gconf-client.h>
+#include <dbus/dbus-glib.h>
 
 #include "gcm-utils.h"
 #include "gcm-clut.h"
@@ -35,6 +36,154 @@
 
 #include "egg-debug.h"
 
+/**
+ * gcm_utils_linkify:
+ **/
+gchar *
+gcm_utils_linkify (const gchar *text)
+{
+	guint i;
+	guint j = 0;
+	gboolean ret;
+	GString *string;
+
+	/* find and replace links */
+	string = g_string_new ("");
+	for (i=0;; i++) {
+
+		/* find the start of a link */
+		ret = g_str_has_prefix (&text[i], "http://");
+		if (ret) {
+			egg_debug ("dump from %i to %i", j, i);
+			g_string_append_len (string, text+j, i-j);
+			for (j=i;; j++) {
+				/* find the end of the link, or the end of the string */
+				if (text[j] == '\0' ||
+				    text[j] == ' ') {
+					egg_debug ("link is from %i to %i", i, j);
+					g_string_append (string, "<a href=\"");
+					g_string_append_len (string, text+i, j-i);
+					g_string_append (string, "\">");
+					g_string_append_len (string, text+i, j-i);
+					g_string_append (string, "</a>");
+					break;
+				}
+			}
+		}
+
+		/* end of the string, dump what's left */
+		if (text[i] == '\0') {
+			egg_debug ("dump from %i to %i", j, i);
+			g_string_append_len (string, text+j, i-j);
+			break;
+		}
+	}
+	return g_string_free (string, FALSE);
+}
+
+/**
+ * gcm_utils_is_icc_profile:
+ **/
+gboolean
+gcm_utils_is_icc_profile (const gchar *filename)
+{
+	GFile *file;
+	GFileInfo *info;
+	const gchar *type;
+	GError *error = NULL;
+	gboolean ret = FALSE;
+
+	/* get content type for file */
+	file = g_file_new_for_path (filename);
+	info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE, G_FILE_QUERY_INFO_NONE, NULL, &error);
+	if (info != NULL) {
+		type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
+		if (g_strcmp0 (type, "application/vnd.iccprofile") == 0) {
+			ret = TRUE;
+			goto out;
+		}
+	} else {
+		egg_warning ("failed to get content type of %s: %s", filename, error->message);
+		g_error_free (error);
+	}
+
+	/* fall back if we have not got a new enought s-m-i */
+	if (g_str_has_suffix (filename, ".icc")) {
+		ret = TRUE;
+		goto out;
+	}
+	if (g_str_has_suffix (filename, ".icm")) {
+		ret = TRUE;
+		goto out;
+	}
+	if (g_str_has_suffix (filename, ".ICC")) {
+		ret = TRUE;
+		goto out;
+	}
+	if (g_str_has_suffix (filename, ".ICM")) {
+		ret = TRUE;
+		goto out;
+	}
+out:
+	if (info != NULL)
+		g_object_unref (info);
+	g_object_unref (file);
+	return ret;
+}
+
+/**
+ * gcm_utils_install_package:
+ **/
+gboolean
+gcm_utils_install_package (const gchar *package_name, GtkWindow *window)
+{
+	DBusGConnection *connection;
+	DBusGProxy *proxy;
+	GError *error = NULL;
+	gboolean ret;
+	guint32 xid = 0;
+	gchar **packages = NULL;
+
+	g_return_val_if_fail (package_name != NULL, FALSE);
+
+#ifndef GCM_USE_PACKAGEKIT
+	egg_warning ("cannot install %s: this package was not compiled with --enable-packagekit", package_name);
+	return FALSE;
+#endif
+
+	/* get xid of this window */
+	if (window != NULL)
+		xid = gdk_x11_drawable_get_xid (gtk_widget_get_window (GTK_WIDGET(window)));
+
+	/* we're expecting an array of packages */
+	packages = g_strsplit (package_name, "|", 1);
+
+	/* get a session bus connection */
+	connection = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+
+	/* connect to PackageKit */
+	proxy = dbus_g_proxy_new_for_name (connection,
+					   "org.freedesktop.PackageKit",
+					   "/org/freedesktop/PackageKit",
+					   "org.freedesktop.PackageKit.Modify");
+
+	/* execute sync method */
+	ret = dbus_g_proxy_call (proxy, "InstallPackageNames", &error,
+				 G_TYPE_UINT, xid,
+				 G_TYPE_STRV, packages,
+				 G_TYPE_STRING, "hide-confirm-search,hide-finished",
+				 G_TYPE_INVALID, G_TYPE_INVALID);
+	if (!ret) {
+		egg_warning ("failed to install package: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+out:
+	g_object_unref (proxy);
+	g_strfreev (packages);
+	return ret;
+}
+
 /**
  * gcm_utils_output_is_lcd_internal:
  * @output_name: the output name
@@ -44,6 +193,7 @@
 gboolean
 gcm_utils_output_is_lcd_internal (const gchar *output_name)
 {
+	g_return_val_if_fail (output_name != NULL, FALSE);
 	if (g_strstr_len (output_name, -1, "LVDS") != NULL)
 		return TRUE;
 	if (g_strstr_len (output_name, -1, "lvds") != NULL)
@@ -60,6 +210,7 @@ gcm_utils_output_is_lcd_internal (const gchar *output_name)
 gboolean
 gcm_utils_output_is_lcd (const gchar *output_name)
 {
+	g_return_val_if_fail (output_name != NULL, FALSE);
 	if (gcm_utils_output_is_lcd_internal (output_name))
 		return TRUE;
 	if (g_strstr_len (output_name, -1, "DVI") != NULL)
@@ -69,6 +220,68 @@ gcm_utils_output_is_lcd (const gchar *output_name)
 	return FALSE;
 }
 
+/**
+ * gcm_utils_ensure_sane_length:
+ **/
+void
+gcm_utils_ensure_sane_length (gchar *text, guint max_length)
+{
+	guint i;
+	guint len;
+
+	g_return_if_fail (text != NULL);
+
+	/* get length */
+	len = strlen (text);
+
+	/* check we have room for ellipsis */
+	if (len <= max_length - 4)
+		return;
+
+	/* already correct len */
+	if (len == max_length)
+		return;
+
+	/* truncate, finding prior word break */
+	for (i=max_length-1; i>0; i--) {
+		if (text[i] == ' ')
+			break;
+	}
+
+	/* one long string with no spaces */
+	if (i == 0)
+		i = max_length - 3;
+
+	/* ellipsis */
+	text[i+0] = '.';
+	text[i+1] = '.';
+	text[i+2] = '.';
+	text[i+3] = '\0';
+}
+
+/**
+ * gcm_utils_ensure_printable:
+ **/
+void
+gcm_utils_ensure_printable (gchar *text)
+{
+	guint i;
+	guint idx = 0;
+
+	g_return_if_fail (text != NULL);
+
+	for (i=0; text[i] != '\0'; i++) {
+		if (g_ascii_isalnum (text[i]) ||
+		    g_ascii_ispunct (text[i]) ||
+		    text[i] == ' ')
+			text[idx++] = text[i];
+	}
+	text[idx] = '\0';
+
+	/* broken profiles have _ instead of spaces */
+	g_strdelimit (text, "_", ' ');
+}
+
 /**
  * gcm_utils_get_gamma_size_fallback:
  **/
@@ -215,17 +428,20 @@ gcm_utils_set_gamma_for_device (GcmDevice *device, GError **error)
 {
 	gboolean ret = FALSE;
 	GcmClut *clut = NULL;
+	GcmProfile *profile = NULL;
 	GcmXserver *xserver = NULL;
 	GnomeRRCrtc *crtc;
 	GnomeRROutput *output;
 	gint x, y;
-	gchar *profile = NULL;
+	gchar *filename = NULL;
+	gchar *filename_systemwide = NULL;
 	gfloat gamma;
 	gfloat brightness;
 	gfloat contrast;
 	gchar *output_name;
 	gchar *id = NULL;
 	guint size;
+	gboolean saved;
 	gboolean use_global;
 	gboolean use_atom;
 	gboolean leftmost_screen = FALSE;
@@ -233,6 +449,8 @@ gcm_utils_set_gamma_for_device (GcmDevice *device, GError **error)
 	GnomeRRScreen *rr_screen = NULL;
 	GConfClient *gconf_client = NULL;
 
+	g_return_val_if_fail (device != NULL, FALSE);
+
 	/* use gconf to decide to set LUT or set ATOMs */
 	gconf_client = gconf_client_get_default ();
 
@@ -240,7 +458,8 @@ gcm_utils_set_gamma_for_device (GcmDevice *device, GError **error)
 	g_object_get (device,
 		      "id", &id,
 		      "type", &type,
-		      "profile-filename", &profile,
+		      "saved", &saved,
+		      "profile-filename", &filename,
 		      "gamma", &gamma,
 		      "brightness", &brightness,
 		      "contrast", &contrast,
@@ -261,6 +480,16 @@ gcm_utils_set_gamma_for_device (GcmDevice *device, GError **error)
 		goto out;
 	}
 
+	/* if not saved, try to find default filename */
+	if (!saved && filename == NULL) {
+		filename_systemwide = g_strdup_printf ("%s/%s.icc", GCM_SYSTEM_PROFILES_DIR, id);
+		ret = g_file_test (filename_systemwide, G_FILE_TEST_EXISTS);
+		if (ret) {
+			egg_error ("using systemwide %s as profile", filename_systemwide);
+			filename = g_strdup (filename_systemwide);
+		}
+	}
+
 	/* check we have an output */
 	rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), NULL, NULL, error);
 	if (rr_screen == NULL)
@@ -285,28 +514,31 @@ gcm_utils_set_gamma_for_device (GcmDevice *device, GError **error)
 	if (size == 0)
 		goto out;
 
-	/* create CLUT */
-	clut = gcm_clut_new ();
-
 	/* only set the CLUT if we're not seting the atom */
 	use_global = gconf_client_get_bool (gconf_client, GCM_SETTINGS_GLOBAL_DISPLAY_CORRECTION, NULL);
+	if (use_global && filename != NULL) {
+		/* create dummy CLUT */
+		profile = gcm_profile_default_new ();
+		ret = gcm_profile_parse (profile, filename, error);
+		if (!ret)
+			goto out;
+
+		/* create a CLUT from the profile */
+		clut = gcm_profile_generate_vcgt (profile, size);
+	} else {
+		/* create dummy CLUT */
+		clut = gcm_clut_new ();
+		g_object_set (clut,
+			      "size", size,
+			      NULL);
+	}
+
+	/* do fine adjustment */
 	if (use_global) {
 		g_object_set (clut,
-			      "profile", profile,
 			      "gamma", gamma,
 			      "brightness", brightness,
 			      "contrast", contrast,
-			      "size", size,
-			      NULL);
-
-		/* load this new profile */
-		ret = gcm_clut_load_from_profile (clut, error);
-		if (!ret)
-			goto out;
-	} else {
-		/* we're using a dummy clut */
-		g_object_set (clut,
-			      "size", size,
 			      NULL);
 	}
 
@@ -324,7 +556,7 @@ gcm_utils_set_gamma_for_device (GcmDevice *device, GError **error)
 
 	/* either remove the atoms or set them */
 	use_atom = gconf_client_get_bool (gconf_client, GCM_SETTINGS_SET_ICC_PROFILE_ATOM, NULL);
-	if (!use_atom || profile == NULL) {
+	if (!use_atom || filename == NULL) {
 
 		/* remove the output atom if there's nothing to show */
 		ret = gcm_xserver_remove_output_profile (xserver, output_name, error);
@@ -339,20 +571,21 @@ gcm_utils_set_gamma_for_device (GcmDevice *device, GError **error)
 		}
 	} else {
 		/* set the per-output and per screen profile atoms */
-		ret = gcm_xserver_set_output_profile (xserver, output_name, profile, error);
+		ret = gcm_xserver_set_output_profile (xserver, output_name, filename, error);
 		if (!ret)
 			goto out;
 
 		/* primary screen */
 		if (leftmost_screen) {
-			ret = gcm_xserver_set_root_window_profile (xserver, profile, error);
+			ret = gcm_xserver_set_root_window_profile (xserver, filename, error);
 			if (!ret)
 				goto out;
 		}
 	}
 out:
 	g_free (id);
-	g_free (profile);
+	g_free (filename);
+	g_free (filename_systemwide);
 	g_free (output_name);
 	if (gconf_client != NULL)
 		g_object_unref (gconf_client);
@@ -360,6 +593,8 @@ out:
 		gnome_rr_screen_destroy (rr_screen);
 	if (clut != NULL)
 		g_object_unref (clut);
+	if (profile != NULL)
+		g_object_unref (profile);
 	if (xserver != NULL)
 		g_object_unref (xserver);
 	return ret;
@@ -433,9 +668,6 @@ gcm_utils_get_profile_filenames (void)
 	gcm_utils_get_profile_filenames_for_directory (array, "/usr/share/color/icc");
 	gcm_utils_get_profile_filenames_for_directory (array, "/usr/local/share/color/icc");
 
-	/* get gnome-color-manager test profiles */
-	gcm_utils_get_profile_filenames_for_directory (array, GCM_DATA "/profiles");
-
 	/* get per-user profiles */
 	user = g_build_filename (g_get_home_dir (), "/.color/icc", NULL);
 	gcm_utils_get_profile_filenames_for_directory (array, user);
@@ -456,6 +688,9 @@ gcm_utils_mkdir_and_copy (const gchar *source, const gchar *destination, GError
 	GFile *destfile;
 	GFile *destpath;
 
+	g_return_val_if_fail (source != NULL, FALSE);
+	g_return_val_if_fail (destination != NULL, FALSE);
+
 	/* setup paths */
 	sourcefile = g_file_new_for_path (source);
 	path = g_path_get_dirname (destination);
@@ -492,6 +727,8 @@ gcm_utils_get_profile_destination (const gchar *filename)
 	gchar *basename;
 	gchar *destination;
 
+	g_return_val_if_fail (filename != NULL, NULL);
+
 	/* get destination filename for this source file */
 	basename = g_path_get_basename (filename);
 	destination = g_build_filename (g_get_home_dir (), GCM_PROFILE_PATH, basename, NULL);
@@ -646,6 +883,20 @@ gcm_utils_device_type_to_profile_type (GcmDeviceType type)
 	return profile_type;
 }
 
+/**
+ * gcm_utils_format_date_time:
+ **/
+gchar *
+gcm_utils_format_date_time (const struct tm *created)
+{
+	gchar buffer[256];
+
+	/* TRANSLATORS: this is the profile creation date strftime format */
+	strftime (buffer, sizeof(buffer), _("%B %e %Y, %I:%M:%S %p"), created);
+
+	return g_strdup (g_strchug (buffer));
+}
+
 /***************************************************************************
  ***                          MAKE CHECK TESTS                           ***
  ***************************************************************************/
@@ -658,6 +909,7 @@ gcm_utils_test (EggTest *test)
 	gboolean ret;
 	GError *error = NULL;
 	GPtrArray *array;
+	gchar *text;
 	gchar *filename;
 	GcmProfileType profile_type;
 	GcmDeviceType device_type;
@@ -665,6 +917,17 @@ gcm_utils_test (EggTest *test)
 	if (!egg_test_start (test, "GcmUtils"))
 		return;
 
+	/************************************************************/
+	egg_test_title (test, "Linkify text");
+	text = gcm_utils_linkify ("http://www.dave.org is text http://www.hughsie.com that needs to be linked to http://www.bbc.co.uk really");
+	if (g_strcmp0 (text, "<a href=\"http://www.dave.org\">http://www.dave.org</a> is text "
+			     "<a href=\"http://www.hughsie.com\">http://www.hughsie.com</a> that needs to be linked to "
+			     "<a href=\"http://www.bbc.co.uk\">http://www.bbc.co.uk</a> really") == 0)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "failed to linkify text: %s", text);
+	g_free (text);
+
 	/************************************************************/
 	egg_test_title (test, "get profile filenames");
 	array = gcm_utils_get_profile_filenames ();
@@ -680,6 +943,13 @@ gcm_utils_test (EggTest *test)
 		egg_test_failed (test, "failed to get filename: %s", filename);
 	g_free (filename);
 
+	/************************************************************/
+	egg_test_title (test, "check is icc profile");
+	filename = egg_test_get_data_file ("bluish.icc");
+	ret = gcm_utils_is_icc_profile (filename);
+	egg_test_assert (test, ret);
+	g_free (filename);
+
 	/************************************************************/
 	egg_test_title (test, "detect LVDS panels");
 	ret = gcm_utils_output_is_lcd_internal ("LVDS1");
@@ -720,6 +990,66 @@ gcm_utils_test (EggTest *test)
 		egg_test_failed (test, "failed to get filename: %s", filename);
 	g_free (filename);
 
+	/************************************************************/
+	egg_test_title (test, "check strip printable");
+	text = g_strdup ("1\r34 67_90");
+	gcm_utils_ensure_printable (text);
+	if (g_strcmp0 (text, "134 67 90") == 0)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "invalid value: %s", text);
+	g_free (text);
+
+	/************************************************************/
+	egg_test_title (test, "check sane length high");
+	text = g_strdup ("1234 67890");
+	gcm_utils_ensure_sane_length (text, 1024);
+	if (g_strcmp0 (text, "1234 67890") == 0)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "invalid value: %s", text);
+	g_free (text);
+
+	/************************************************************/
+	egg_test_title (test, "check sane length limit");
+	text = g_strdup ("1234 67890");
+	gcm_utils_ensure_sane_length (text, 10);
+	if (g_strcmp0 (text, "1234 67890") == 0)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "invalid value: %s", text);
+	g_free (text);
+
+	/************************************************************/
+	egg_test_title (test, "check sane length truncate");
+	text = g_strdup ("1234 67890");
+	gcm_utils_ensure_sane_length (text, 8);
+	if (g_strcmp0 (text, "1234...") == 0)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "invalid value: %s", text);
+	g_free (text);
+
+	/************************************************************/
+	egg_test_title (test, "check sane length no spaces");
+	text = g_strdup ("1234 67890");
+	gcm_utils_ensure_sane_length (text, 4);
+	if (g_strcmp0 (text, "1...") == 0)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "invalid value: %s", text);
+	g_free (text);
+
+	/************************************************************/
+	egg_test_title (test, "check sane length no data");
+	text = g_strdup ("");
+	gcm_utils_ensure_sane_length (text, 4);
+	if (g_strcmp0 (text, "") == 0)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "invalid value: %s", text);
+	g_free (text);
+
 	/************************************************************/
 	egg_test_title (test, "get default config location (when in make check)");
 	filename = gcm_utils_get_default_config_location ();
